/* sane - Scanner Access Now Easy.
   Copyright (C) 1997 David Mosberger-Tang
   This file is part of the SANE package.

   SANE is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   SANE is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with sane; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   This file implements a SANE backend for the Connectix QuickCam.  At
   present, only the color camera is supported though the driver
   should be able to easily accommodate black and white cameras.

   Portions of this code are derived from Scott Laird's qcam driver.
   It's copyright notice is reproduced here:

   Copyright (C) 1996 by Scott Laird

   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation
   files (the "Software"), to deal in the Software without
   restriction, including without limitation the rights to use, copy,
   modify, merge, publish, distribute, sublicense, and/or sell copies
   of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be
   included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
   NONINFRINGEMENT.  IN NO EVENT SHALL SCOTT LAIRD BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
   WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */

#include <config.h>

#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <math.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sane.h>
#include <sanei.h>
#include <saneopts.h>
#include <qcam.h>

#define PATH_QCAM_CONFIG	PATH_SANE_CONFIG_DIR "/qcam.conf"

/* status bits */
#define NeedRamTable		(1 << 1)
#define BlackBalanceInProgress	(1 << 6)
#define CameraNotReady		(1 << 7)

/* lpdata bits: */
#define Cmd0_7		0xff
#define CamRdy2		(   1 << 0)	/* byte mode */
#define Data0_6		(0x7f << 1)	/* byte mode */

/* lpstatus bits: */
#define CamRdy1		(   1 << 3)	/* nibble mode */
#define Nibble0_3	(0x0f << 4)	/* nibble mode */
#define Data7_11	(0x1f << 3)	/* byte mode */

/* lpcontrol bits: */
#define Strobe		(   1 << 0)	/* unused */
#define Autofeed	(   1 << 1)
#define Reset_N		(   1 << 2)
#define PCAck		(   1 << 3)
#define BiDir		(   1 << 5)

#define NELEMS(a)	((int) sizeof(a)/sizeof(a[0]))

DBG_DECL(QCAM)

static int num_devices;
static QC_Device * first_dev;
static QC_Scanner * first_handle;

static const SANE_String_Const resolution_list[] =
  {
    "Low",	/* million-mode */
    "High",	/* billion-mode */
    0
  };

static const SANE_Int mono_depth_list[] =
  {
    2,				/* # of elements */
    4, 6
  };

static const SANE_Int color_depth_list[] =
  {
    /*2*/ 1,			/* # of elements */
    /*16,*/ 24			/* "thousand" mode not implemented yet */
  };

static const SANE_Int xfer_scale_list[] =
  {
    3,				/* # of elements */
    1, 2, 4
  };

static const SANE_Range u8_range =
  {
    /* min, max, quantization */
    0, 255, 0
  };

static const SANE_Range x_range[] =
  {
    /* min, max, quantization */
    {0, 338, 2}, /* million mode */
    {0, 676, 4}, /* billion mode */
  };

static const SANE_Range odd_x_range[] =
  {
    /* min, max, quantization */
    {1, 339, 2}, /* million mode */
    {3, 683, 4}, /* billion mode */
  };

static const SANE_Range y_range[] =
  {
    /* min, max, quantization */
    {0,	249, 1}, /* million mode */
    {0,	498, 2}, /* billion mode */
  };

static const SANE_Range odd_y_range[] =
  {
    /* min, max, quantization */
    {0,	249, 1}, /* million mode */
    {1,	499, 2}, /* billion mode */
  };

static size_t
max_string_size (const SANE_String_Const strings[])
{
  size_t size, max_size = 0;
  int i;

  for (i = 0; strings[i]; ++i)
    {
      size = strlen (strings[i]) + 1;
      if (size > max_size)
	max_size = size;
    }
  return max_size;
}

#ifdef __linux__

#if __GNU_LIBRARY__ - 0 < 6
# include <asm/io.h>
#else
# include <sys/io.h>
#endif

#define read_lpdata(d)		inb ((d)->port)
#define read_lpstatus(d)	inb ((d)->port + 1)
#define read_lpcontrol(d)	inb ((d)->port + 2)
#define write_lpdata(d,v)	outb ((v), (d)->port)
#define write_lpcontrol(d,v)	outb ((v), (d)->port + 2)

#endif /* __linux__ */

static SANE_Status
enable_ports (QC_Device * q)
{
  /* better safe than sorry */
  if (q->port < 0x278 || q->port > 0x3bc)
    return SANE_STATUS_INVAL;

  if (ioperm (q->port, 3, 1) < 0)
    return SANE_STATUS_INVAL;

  return SANE_STATUS_GOOD;
}

static SANE_Status
disable_ports (QC_Device *q)
{
  if (ioperm (q->port, 3, 0) < 0)
    return SANE_STATUS_INVAL;

  return SANE_STATUS_GOOD;
}

/* We need a short delay loop -- somthing well under a millisecond.
   Unfortunately, adding 2 usleep(1)'s to qc_command slowed it down by
   a factor of over 1000 over the same loop with 2 usleep(0)'s, and
   that's too slow -- qc_start was taking over a second to run.  This
   seems to help, but if anyone has a good speed-independent pause
   routine, please tell me. -- Scott

   If you're worried about hogging the CPU: don't worry, the qcam
   interface leaves you no choice, so this doesn't make the situation
   any worse... */

static void
qc_wait (QC_Device * q)
{
  read_lpstatus (q);
}


/* This function uses POSIX fcntl-style locking on a file created in
   the /tmp directory.  Because it uses the Unix record locking
   facility, locks are relinquished automatically on process
   termination, so "dead locks" are not a problem.  (FYI, the lock
   file will remain after process termination, but this is actually
   desired so that the next process need not re-creat(2)e it... just
   lock it.)  The wait argument indicates whether or not this funciton
   should "block" waiting for the previous lock to be relinquished.
   This is ideal so that multiple processes (eg. qcam) taking
   "snapshots" can peacefully coexist.

   -- Dave Plonka (plonka@carroll1.cc.edu) */
static SANE_Status
qc_lock_wait (QC_Device * q, int wait)
{
  struct flock sfl;

  DBG(QCAM, 3, "qc_lock_wait: acquiring lock for 0x%x\n", q->port);

  memset (&sfl, 0, sizeof (sfl));

  if (q->lock_fd < 0)
    {
      char lockfile[128];

      sprintf (lockfile, "/tmp/.qcam-%x-lock", q->port);
      q->lock_fd = open (lockfile, O_WRONLY | O_CREAT, 0666);
      if (q->lock_fd < 0)
	{
	  DBG(QCAM, 1, "qc_lock_wait: failed to open %s (%s)\n",
	      lockfile, strerror (errno));
	  return SANE_STATUS_INVAL;
	}

    }

  sfl.l_type = F_WRLCK;
  if (fcntl (q->lock_fd, wait ? F_SETLKW : F_SETLK, &sfl) != 0)
    {
      DBG(QCAM, 1, "qc_lock_wait: failed to acquire lock (%s)\n",
	  strerror (errno));
      return SANE_STATUS_INVAL;
    }

  DBG(QCAM, 3, "qc_lock_wait: got lock for 0x%x\n", q->port);
  return SANE_STATUS_GOOD;
}

static SANE_Status
qc_unlock (QC_Device *q)
{
  struct flock sfl;

  DBG(QCAM, 3, "qc_lock_wait: releasing lock for 0x%x\n", q->port);

  memset (&sfl, 0, sizeof (sfl));

  if (q->lock_fd < 0)
    return SANE_STATUS_INVAL;		 /* port was not locked */

  /* clear the exclusive lock */
  sfl.l_type = F_UNLCK;
  if (fcntl (q->lock_fd, F_SETLK, &sfl) != 0)
    {
      DBG(QCAM, 1, "qc_lock_wait: failed to release lock (%s)\n",
	  strerror (errno));
      return SANE_STATUS_INVAL;
    }
  return SANE_STATUS_GOOD;
}

static SANE_Status
qc_lock (QC_Device *q)
{
   return qc_lock_wait (q, 1);
}

/* Busy-waits for a handshake signal from the QuickCam.  Almost all
   communication with the camera requires handshaking.  This is why
   qcam is a CPU hog.  */
static int
qc_waithand (QC_Device *q, int val)
{
  int status;

  while (((status = read_lpstatus (q)) & CamRdy1) != val);
  return status;
}

/* This is used when the qcam is in bidirectional ("byte") mode, and
   the handshaking signal is CamRdy2 (bit 0 of data reg) instead of
   CamRdy1 (bit 3 of status register).  It also returns the last value
   read, since this data is useful.  */
static unsigned int
qc_waithand2 (QC_Device *q, int val)
{
  unsigned int status;

  do {
    status = read_lpdata (q);
  } while ((status & CamRdy2) != val);
  return status;
}

static unsigned int
qc_send (QC_Device *q, unsigned int byte)
{
  unsigned int echo;
  int n1, n2;

  write_lpdata (q, byte);
  qc_wait (q);
  write_lpcontrol (q, Autofeed | Reset_N);
  qc_wait (q);

  n1 = qc_waithand (q, CamRdy1);

  write_lpcontrol (q, Autofeed | Reset_N | PCAck);
  qc_wait (q);
  n2 = qc_waithand (q, 0);

  echo = (n1 & 0xf0) | ((n2 & 0xf0) >> 4);
#ifndef NDEBUG
  if (echo != byte)
    {
      DBG(QCAM, 1, "qc_send: sent 0x%02x, camera echoed 0x%02x\n", byte, echo);
      n2 = read_lpstatus (q);
      echo = (n1 & 0xf0) | ((n2 & 0xf0) >> 4);
      if (echo != byte)
	DBG(QCAM, 1, "qc_send: (re-read does not help)\n");
      else
	DBG(QCAM, 1, "qc_send: (fixed on re-read)\n");
    }
#endif
  return echo;
}

static int
qc_readparam (QC_Device *q)
{
  int n1, n2;
  int cmd;

  write_lpcontrol (q, Autofeed | Reset_N);		/* clear PCAck */
  n1 = qc_waithand (q, CamRdy1);

  write_lpcontrol (q, Autofeed | Reset_N | PCAck);	/* set PCAck */
  n2 = qc_waithand (q, 0);

  cmd = (n1 & 0xf0) | ((n2 & 0xf0) >> 4);
  return cmd;
}

unsigned int
qc_getstatus (QC_Device *q)
{
  unsigned int status;

  qc_send (q, QC_SEND_STATUS);
  status = qc_readparam (q);
  DBG(QCAM, 3, "qc_getstatus: status=0x%02x\n", status);
  return status;
}

static void
qc_setscanmode (QC_Scanner *s)
{
  QC_Device *q = s->hw;
  unsigned int mode = 0;

  if (q->version == QC_MONO)
    {
      switch (s->val[OPT_XFER_SCALE].w)
	{
	case 1: mode = 0; break;
	case 2: mode = 4; break;
	case 4: mode = 8; break;
	}
      switch (s->val[OPT_DEPTH].w)
	{
	case  4: break;
	case  6: mode +=  2; break;
	}
    }
  else
    {
      switch (s->val[OPT_XFER_SCALE].w)
	{
	case 1: mode = 0; break;
	case 2: mode = 2; break;
	case 4: mode = 4; break;
	}
      if (s->resolution == QC_RES_LOW)
	mode |= 0x18;	/* millions mode */
      else
	mode |= 0x10;	/* billions mode */
    }
  if (s->val[OPT_TEST].w)
    mode |= 0x40;		/* test mode */

  if (q->port_mode == QC_BIDIR)
    mode |= 1;

  DBG(QCAM, 2, "qcam.scanmode (before increment): 0x%x\n", mode);

  if (q->version == QC_COLOR)
    ++mode;

  s->mode = mode;
}

/* Read data bytes from the camera.  The number of bytes read is
   returned as the function result.  Depending on the mode, it may be
   either 1, 3 or 6.  On failure, 0 is returned.  If buffer is 0, the
   internal state-machine is reset.  */
static size_t
qc_readbytes (QC_Scanner *s, unsigned char buffer[])
{
  QC_Device * q = s->hw;
  unsigned int hi, lo;
  unsigned int hi2, lo2;
  size_t bytes = 0;

  if (!buffer)
    {
      s->readbytes_state = 0;
      return 0; 
    }

  switch (q->port_mode)
    {
    case QC_BIDIR:
      /* bi-directional port */

      /* read off 24 bits: */
      write_lpcontrol (q, Autofeed | Reset_N | BiDir);
      lo = qc_waithand2 (q, 1) >> 1;
      hi = (read_lpstatus (q) >> 3) & 0x1f;
      write_lpcontrol (q, Autofeed | Reset_N | PCAck | BiDir);
      lo2 = qc_waithand2 (q, 0) >> 1;
      hi2 = (read_lpstatus (q) >> 3) & 0x1f;
      if (q->version == QC_COLOR)
	{
	  /* is Nibble3 inverted for color quickcams only? */
	  hi  ^= 0x10;
	  hi2 ^= 0x10;
	}
      switch (s->val[OPT_DEPTH].w)
	{
	case 4:
	  buffer[0] = lo & 0xf;
	  buffer[1] = ((lo & 0x70) >> 4) | ((hi & 1) << 3);
	  buffer[2] = (hi & 0x1e) >> 1;
	  buffer[3] = lo2 & 0xf;
	  buffer[4] = ((lo2 & 0x70) >> 4) | ((hi2 & 1) << 3);
	  buffer[5] = (hi2 & 0x1e) >> 1;
	  bytes = 6;
	  break;

	case 6:
	  buffer[0] = lo & 0x3f;
	  buffer[1] = ((lo & 0x40) >> 6) | (hi << 1);
	  buffer[2] = lo2 & 0x3f;
	  buffer[3] = ((lo2 & 0x40) >> 6) | (hi2 << 1);
	  bytes = 4;
	  break;

	case 24:
	  buffer[0] = lo | ((hi & 0x1) << 7);
	  buffer[1] = ((hi2 & 0x1e) >> 1) | ((hi & 0x1e) << 3);
	  buffer[2] = lo2 | ((hi2 & 0x1) << 7);
	  bytes = 3;
	  break;
	}
      break;

    case QC_UNIDIR:  /* Unidirectional Port */
      write_lpcontrol (q, Autofeed | Reset_N);
      lo = (qc_waithand (q, CamRdy1) & 0xf0) >> 4;
      write_lpcontrol (q, Autofeed | Reset_N | PCAck);
      hi = (qc_waithand (q,0) & 0xf0) >> 4;

      if (q->version == QC_COLOR)
	{
	  /* invert Nibble3 */
	  hi ^= 8;
	  lo ^= 8;
	}

      switch (s->val[OPT_DEPTH].w)
	{
	case 4:
	  buffer[0] = lo;
	  buffer[1] = hi;
	  bytes = 2;
	  break;

	case 6:
	  switch (s->readbytes_state)
	    {
	    case 0:
	      buffer[0] = (lo << 2) | ((hi & 0xc) >> 2);
	      s->saved_bits = (hi & 3) << 4;
	      s->readbytes_state = 1;
	      bytes = 1;
	      break;

	    case 1:
	      buffer[0] = lo | s->saved_bits;
	      s->saved_bits = hi << 2;
	      s->readbytes_state = 2;
	      bytes = 1;
	      break;

	    case 2:
	      buffer[0] = ((lo & 0xc) >> 2) | s->saved_bits;
	      buffer[1] = ((lo & 3) << 4)  | hi;
	      s->readbytes_state = 0;
	      bytes = 2;
	      break;

	    default:
	      DBG(QCAM, 1, "qc_readbytes: bad unidir 6-bit stat %d\n",
		  s->readbytes_state);
	      break;
	    }
	  break;

	case 24:
	  buffer[0] = (lo << 4) | hi;
	  bytes = 1;
	  break;

	default:
	  DBG(QCAM, 1, "qc_readbytes: bad unidir bit depth %d\n",
	      s->val[OPT_DEPTH].w);
	  break;
	}
      break;

    default:
      DBG(QCAM, 1, "qc_readbytes: bad port_mode %d\n", q->port_mode);
      break;
    }
  return bytes;
}

SANE_Status
attach (const char *devname, QC_Device **devp)
{
  int i, n1, n2, s1, s2, cmd, port;
  SANE_String model_name;
  SANE_Status result;
  QC_Device * q;
  char *endp;

  errno = 0;
  port = strtol (devname, &endp, 0);
  if (endp == devname || errno == ERANGE)
    {
      DBG(QCAM, 1, "qcam.attach: invalid port address `%s'\n", devname);
      return SANE_STATUS_INVAL;
    }

  for (q = first_dev; q; q = q->next)
    if (port == q->port)
      {
	if (devp)
	  *devp = q;
	return SANE_STATUS_GOOD;
      }

  q = malloc (sizeof (*q));
  if (!q)
    return SANE_STATUS_NO_MEM;

  memset (q, 0, sizeof (*q));
  q->port = port;
  q->lock_fd = -1;

  result = enable_ports (q);
  if (result != SANE_STATUS_GOOD)
    {
      DBG(QCAM, 1, "qcam.attach: cannot enable ports (%s)\n",
	  strerror (errno));
      free (q);
      return SANE_STATUS_INVAL;
    }

  /* lock camera while we determine its version: */
  qc_lock (q);

  /* reset camera: */
  write_lpcontrol (q, Strobe | Autofeed | Reset_N | PCAck);
  write_lpcontrol (q, Strobe | Autofeed | PCAck);
  write_lpcontrol (q, Strobe | Autofeed | Reset_N | PCAck);

  write_lpdata (q, QC_SEND_VERSION);
  write_lpcontrol (q, Autofeed | Reset_N);	/* make PCAck inactive */

  for (i = 0; (i < 1000) && !(s1 = (n1 = read_lpstatus (q)) & CamRdy1); i++);
  if (!s1)
    goto unlock_and_fail;

  write_lpcontrol (q, Autofeed | Reset_N | PCAck);

  for (i = 0; (i < 1000) && (s2 = (n2 = read_lpstatus (q)) & CamRdy1); i++);
  if (s2)
    goto unlock_and_fail;

  cmd = (n1 & 0xf0) | ((n2 & 0xf0) >> 4);
 
  if (cmd != QC_SEND_VERSION)
    goto unlock_and_fail;

  q->version = qc_readparam (q);

  write_lpcontrol (q, BiDir);
  write_lpdata (q, 0x75);
  if (read_lpdata (q) != 0x75)
    q->port_mode = QC_BIDIR;
  else
    q->port_mode = QC_UNIDIR;

  qc_unlock (q);

  q->sane.name   = strdup (devname);
  q->sane.vendor = "Connectix";
  q->sane.model  =
    (q->version == QC_COLOR) ? "Color QuickCam" : "B&W QuickCam";
  q->sane.type   = "video camera";

  ++num_devices;
  q->next = first_dev;
  first_dev = q;

  if (devp)
    *devp = q;
  return SANE_STATUS_GOOD;

unlock_and_fail:
  qc_unlock (q);
  free (q);
  return SANE_STATUS_INVAL;
}

static SANE_Status
init_options (QC_Scanner *s)
{
  int i;

  memset (s->opt, 0, sizeof (s->opt));
  memset (s->val, 0, sizeof (s->val));

  for (i = 0; i < NUM_OPTIONS; ++i)
    {
      s->opt[i].size = sizeof (SANE_Word);
      s->opt[i].cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
    }

  s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
  s->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
  s->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
  s->val[OPT_NUM_OPTS].w = NUM_OPTIONS;

  /* "Mode" group: */

  s->opt[OPT_MODE_GROUP].title = "Scan Mode";
  s->opt[OPT_MODE_GROUP].desc = "";
  s->opt[OPT_MODE_GROUP].type = SANE_TYPE_GROUP;
  s->opt[OPT_MODE_GROUP].cap = 0;
  s->opt[OPT_MODE_GROUP].constraint_type = SANE_CONSTRAINT_NONE;

  /* resolution */
  s->opt[OPT_RESOLUTION].name = SANE_NAME_SCAN_RESOLUTION;
  s->opt[OPT_RESOLUTION].title = SANE_TITLE_SCAN_RESOLUTION;
  s->opt[OPT_RESOLUTION].desc = SANE_DESC_SCAN_RESOLUTION;
  s->opt[OPT_RESOLUTION].type = SANE_TYPE_STRING;
  s->opt[OPT_RESOLUTION].unit = SANE_UNIT_NONE;
  s->opt[OPT_RESOLUTION].constraint_type = SANE_CONSTRAINT_STRING_LIST;
  s->opt[OPT_RESOLUTION].constraint.string_list = resolution_list;
  s->val[OPT_RESOLUTION].s = strdup (resolution_list[QC_RES_LOW]);

  /* bit-depth */
  s->opt[OPT_DEPTH].name = SANE_NAME_BIT_DEPTH;
  s->opt[OPT_DEPTH].title = "Pixel depth";
  s->opt[OPT_DEPTH].desc = "Number of bits per pixel.";
  s->opt[OPT_DEPTH].type = SANE_TYPE_INT;
  s->opt[OPT_DEPTH].unit = SANE_UNIT_BIT;
  s->opt[OPT_DEPTH].constraint_type = SANE_CONSTRAINT_WORD_LIST;
  s->opt[OPT_DEPTH].constraint.word_list = mono_depth_list;
  s->val[OPT_DEPTH].w = mono_depth_list[NELEMS(mono_depth_list) - 1];

  /* test */
  s->opt[OPT_TEST].name = "test-image";
  s->opt[OPT_TEST].title = "Force test image";
  s->opt[OPT_TEST].desc =
    "Acquire a test-image instead of the image seen by the camera. "
    "The test image consists of red, green, and blue squares of "
    "32x32 pixels each.  Use this to find out whether the "
    "camera is connected properly.";
  s->opt[OPT_TEST].type = SANE_TYPE_BOOL;
  s->val[OPT_TEST].w = SANE_FALSE;

  /* "Geometry" group: */
  s->opt[OPT_GEOMETRY_GROUP].title = "Geometry";
  s->opt[OPT_GEOMETRY_GROUP].desc = "";
  s->opt[OPT_GEOMETRY_GROUP].type = SANE_TYPE_GROUP;
  s->opt[OPT_GEOMETRY_GROUP].cap = 0;
  s->opt[OPT_GEOMETRY_GROUP].constraint_type = SANE_CONSTRAINT_NONE;

  /* top-left x */
  s->opt[OPT_TL_X].name = SANE_NAME_SCAN_TL_X;
  s->opt[OPT_TL_X].title = SANE_TITLE_SCAN_TL_X;
  s->opt[OPT_TL_X].desc = SANE_DESC_SCAN_TL_X;
  s->opt[OPT_TL_X].type = SANE_TYPE_INT;
  s->opt[OPT_TL_X].unit = SANE_UNIT_PIXEL;
  s->opt[OPT_TL_X].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_TL_X].constraint.range = &x_range[QC_RES_LOW];
  s->val[OPT_TL_X].w = 10;

  /* top-left y */
  s->opt[OPT_TL_Y].name = SANE_NAME_SCAN_TL_Y;
  s->opt[OPT_TL_Y].title = SANE_TITLE_SCAN_TL_Y;
  s->opt[OPT_TL_Y].desc = SANE_DESC_SCAN_TL_Y;
  s->opt[OPT_TL_Y].type = SANE_TYPE_INT;
  s->opt[OPT_TL_Y].unit = SANE_UNIT_PIXEL;
  s->opt[OPT_TL_Y].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_TL_Y].constraint.range = &y_range[QC_RES_LOW];
  s->val[OPT_TL_Y].w = 0;

  /* bottom-right x */
  s->opt[OPT_BR_X].name = SANE_NAME_SCAN_BR_X;
  s->opt[OPT_BR_X].title = SANE_TITLE_SCAN_BR_X;
  s->opt[OPT_BR_X].desc = SANE_DESC_SCAN_BR_X;
  s->opt[OPT_BR_X].type = SANE_TYPE_INT;
  s->opt[OPT_BR_X].unit = SANE_UNIT_PIXEL;
  s->opt[OPT_BR_X].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_BR_X].constraint.range = &odd_x_range[QC_RES_LOW];
  s->val[OPT_BR_X].w = 339;

  /* bottom-right y */
  s->opt[OPT_BR_Y].name = SANE_NAME_SCAN_BR_Y;
  s->opt[OPT_BR_Y].title = SANE_TITLE_SCAN_BR_Y;
  s->opt[OPT_BR_Y].desc = SANE_DESC_SCAN_BR_Y;
  s->opt[OPT_BR_Y].type = SANE_TYPE_INT;
  s->opt[OPT_BR_Y].unit = SANE_UNIT_PIXEL;
  s->opt[OPT_BR_Y].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_BR_Y].constraint.range = &odd_y_range[QC_RES_LOW];
  s->val[OPT_BR_Y].w = 245;

  /* xfer-scale */
  s->opt[OPT_XFER_SCALE].name = "transfer-scale";
  s->opt[OPT_XFER_SCALE].title = "Transfer scale";
  s->opt[OPT_XFER_SCALE].desc =
    "The transferscale determines how many of the acquired pixels actually "
    "get sent to the computer.  For example, a transfer scale of 2 would "
    "request that every other acquired pixel would be omitted.  That is, "
    "when scanning a 200 pixel wide and 100 pixel tall area, the resulting "
    "image would be only 100x50 pixels large.  Using a large transfer scale "
    "improves acquisition speed, but reduces resolution.";
  s->opt[OPT_XFER_SCALE].type = SANE_TYPE_INT;
  s->opt[OPT_XFER_SCALE].constraint_type = SANE_CONSTRAINT_WORD_LIST;
  s->opt[OPT_XFER_SCALE].constraint.word_list = xfer_scale_list;
  s->val[OPT_XFER_SCALE].w = xfer_scale_list[1];


  /* "Enhancement" group: */

  s->opt[OPT_ENHANCEMENT_GROUP].title = "Enhancement";
  s->opt[OPT_ENHANCEMENT_GROUP].desc = "";
  s->opt[OPT_ENHANCEMENT_GROUP].type = SANE_TYPE_GROUP;
  s->opt[OPT_ENHANCEMENT_GROUP].cap = 0;
  s->opt[OPT_ENHANCEMENT_GROUP].constraint_type = SANE_CONSTRAINT_NONE;

  /* brightness */
  s->opt[OPT_BRIGHTNESS].name = SANE_NAME_BRIGHTNESS;
  s->opt[OPT_BRIGHTNESS].title = SANE_TITLE_BRIGHTNESS;
  s->opt[OPT_BRIGHTNESS].desc = SANE_DESC_BRIGHTNESS
    "  In a conventional camera, this control corresponds to the "
    "exposure time.";
  s->opt[OPT_BRIGHTNESS].type = SANE_TYPE_INT;
  s->opt[OPT_BRIGHTNESS].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_BRIGHTNESS].constraint.range = &u8_range;
  s->val[OPT_BRIGHTNESS].w = 135;

  /* contrast */
  s->opt[OPT_CONTRAST].name = SANE_NAME_CONTRAST;
  s->opt[OPT_CONTRAST].title = SANE_TITLE_CONTRAST;
  s->opt[OPT_CONTRAST].desc = SANE_DESC_CONTRAST;
  s->opt[OPT_CONTRAST].type = SANE_TYPE_INT;
  s->opt[OPT_CONTRAST].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_CONTRAST].constraint.range = &u8_range;
  s->val[OPT_CONTRAST].w = 104;

  /* black-level */
  s->opt[OPT_BLACK_LEVEL].name = SANE_NAME_BLACK_LEVEL;
  s->opt[OPT_BLACK_LEVEL].title = SANE_TITLE_BLACK_LEVEL;
  s->opt[OPT_BLACK_LEVEL].desc = SANE_DESC_BLACK_LEVEL
    " This value should be selected so that black areas just start "
    "to look really black (not gray).";
  s->opt[OPT_BLACK_LEVEL].type = SANE_TYPE_INT;
  s->opt[OPT_BLACK_LEVEL].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_BLACK_LEVEL].constraint.range = &u8_range;
  s->val[OPT_BLACK_LEVEL].w = 0;

  /* white-level */
  s->opt[OPT_WHITE_LEVEL].name = SANE_NAME_WHITE_LEVEL;
  s->opt[OPT_WHITE_LEVEL].title = SANE_TITLE_WHITE_LEVEL;
  s->opt[OPT_WHITE_LEVEL].desc = SANE_DESC_WHITE_LEVEL
    " This value should be selected so that white areas just start "
    "to look really white (not gray).";
  s->opt[OPT_WHITE_LEVEL].type = SANE_TYPE_INT;
  s->opt[OPT_WHITE_LEVEL].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_WHITE_LEVEL].constraint.range = &u8_range;
  s->val[OPT_WHITE_LEVEL].w = 150;

  /* hue */
  s->opt[OPT_HUE].name = SANE_NAME_HUE;
  s->opt[OPT_HUE].title = SANE_TITLE_HUE;
  s->opt[OPT_HUE].desc = SANE_DESC_HUE;
  s->opt[OPT_HUE].type = SANE_TYPE_INT;
  s->opt[OPT_HUE].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_HUE].constraint.range = &u8_range;
  s->val[OPT_HUE].w = 128;

  /* saturation */
  s->opt[OPT_SATURATION].name = SANE_NAME_SATURATION;
  s->opt[OPT_SATURATION].title = SANE_TITLE_SATURATION;
  s->opt[OPT_SATURATION].desc = SANE_DESC_SATURATION;
  s->opt[OPT_SATURATION].type = SANE_TYPE_INT;
  s->opt[OPT_SATURATION].constraint_type = SANE_CONSTRAINT_RANGE;
  s->opt[OPT_SATURATION].constraint.range = &u8_range;
  s->val[OPT_SATURATION].w = 100;

  return SANE_STATUS_GOOD;
}

SANE_Status
sane_init (SANE_Int *version_code)
{
  char dev_name[PATH_MAX], * str;
  size_t len;
  FILE *fp;
  int port;

  DBG_INIT(QCAM);

  if (version_code)
    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);

  fp = fopen (PATH_QCAM_CONFIG, "r");
  if (!fp)
    {
      DBG(QCAM, 1, "sane_init: file `%s' not accessible\n", PATH_QCAM_CONFIG);
      return SANE_STATUS_INVAL;
    }

  while (fgets (dev_name, sizeof (dev_name), fp))
    {
      if (dev_name[0] == '#')		/* ignore line comments */
	continue;
      len = strlen (dev_name);
      if (dev_name[len - 1] == '\n')
	dev_name[--len] = '\0';

      if (!len)
	continue;			/* ignore empty lines */

      for (str = dev_name; *str && !isblank (*str) && *str != '#'; ++str);
      *str = '\0';

      attach (dev_name, 0);
    }
  fclose (fp);
  return SANE_STATUS_GOOD;
}

void
sane_exit (void)
{
  QC_Device *dev, *next;

  for (dev = first_dev; dev; dev = next)
    {
      next = dev->next;
      free ((void *) dev->sane.name);
      free (dev);
    }
}

SANE_Status
sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
{
  static const SANE_Device ** devlist = 0;
  QC_Device *dev;
  int i;

  if (devlist)
    free (devlist);

  devlist = malloc ((num_devices + 1) * sizeof (devlist[0]));
  if (!devlist)
    return SANE_STATUS_NO_MEM;

  i = 0;
  for (dev = first_dev; i < num_devices; dev = dev->next)
    devlist[i++] = &dev->sane;
  devlist[i++] = 0;

  *device_list = devlist;
  return SANE_STATUS_GOOD;
}

SANE_Status
sane_open (SANE_String_Const devicename, SANE_Handle *handle)
{
  SANE_Status status;
  QC_Device *dev;
  QC_Scanner *s;

  if (devicename[0])
    {
      status = attach (devicename, &dev);
      if (status != SANE_STATUS_GOOD)
	return status;
    }
  else
    /* empty devicname -> use first device */
    dev = first_dev;

  if (!dev)
    return SANE_STATUS_INVAL;

  s = malloc (sizeof (*s));
  if (!s)
    return SANE_STATUS_NO_MEM;
  memset (s, 0, sizeof (*s));
  s->user_corner = 0;
  s->previous_black_level = -1;
  s->hw = dev;

  init_options (s);

  /* The contrast option seems to have an effect for b&w cameras only,
     so don't give the user the impression that this is a useful thing
     to set... */
  if (s->hw->version == QC_COLOR)
    {
      s->opt[OPT_CONTRAST].cap |= SANE_CAP_INACTIVE;
      s->opt[OPT_DEPTH].constraint.word_list = color_depth_list;
      s->val[OPT_DEPTH].w = color_depth_list[NELEMS(color_depth_list) - 1];
    }

  /* insert newly opened handle into list of open handles: */
  s->next = first_handle;
  first_handle = s;

  *handle = s;
  return SANE_STATUS_GOOD;
}

void
sane_close (SANE_Handle handle)
{
  QC_Scanner *prev, *s;

  /* remove handle from list of open handles: */
  prev = 0;
  for (s = first_handle; s; s = s->next)
    {
      if (s == handle)
	break;
      prev = s;
    }
  if (!s)
    return;		/* oops, not a handle we know about */

  if (s->scanning)
    sane_cancel (handle);

  if (s->buffer)
    free (s->buffer);

  if (prev)
    prev->next = s->next;
  else
    first_handle = s;

  free (handle);
}

const SANE_Option_Descriptor *
sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
{
  QC_Scanner *s = handle;

  if ((unsigned) option >= NUM_OPTIONS)
    return 0;
  return s->opt + option;
}

SANE_Status
sane_control_option (SANE_Handle handle, SANE_Int option,
		     SANE_Action action, void *val, SANE_Int *info)
{
  QC_Scanner *s = handle;
  SANE_Status status;
  SANE_Word cap;

  if (info)
    *info = 0;

  if (s->scanning || option >= NUM_OPTIONS)
    return SANE_STATUS_INVAL;

  cap = s->opt[option].cap;

  if (!SANE_OPTION_IS_ACTIVE (cap))
    return SANE_STATUS_INVAL;

  if (action == SANE_ACTION_GET_VALUE)
    {
      switch (option)
	{
	  /* word options: */
	case OPT_NUM_OPTS:
	case OPT_DEPTH:
	case OPT_TEST:
	case OPT_TL_X:
	case OPT_TL_Y:
	case OPT_BR_X:
	case OPT_BR_Y:
	case OPT_XFER_SCALE:
	case OPT_BRIGHTNESS:
	case OPT_CONTRAST:
	case OPT_BLACK_LEVEL:
	case OPT_WHITE_LEVEL:
	case OPT_HUE:
	case OPT_SATURATION:
	  *(SANE_Word *) val = s->val[option].w;
	  return SANE_STATUS_GOOD;

	  /* string options: */
	case OPT_RESOLUTION:
	  strcpy (val, s->val[option].s);
	  return SANE_STATUS_GOOD;

	default:
	  DBG(QCAM, 1, "qcam.control_option: option %d unknown\n", option);
	}
    }
  else if (action == SANE_ACTION_SET_VALUE)
    {
      if (!SANE_OPTION_IS_SETTABLE (cap))
	return SANE_STATUS_INVAL;

      status = sanei_constrain_value (s->opt + option, val, info);
      if (status != SANE_STATUS_GOOD)
	return status;

      if (option >= OPT_TL_X && option <= OPT_BR_Y)
	s->user_corner |= 1 << option - OPT_TL_X;

      switch (option)
	{
	  /* (mostly) side-effect-free word options: */
	case OPT_TL_X:
	case OPT_TL_Y:
	case OPT_BR_X:
	case OPT_BR_Y:
	case OPT_XFER_SCALE:
	  if (info)
	    *info |= SANE_INFO_RELOAD_PARAMS;
	  /* fall through */
	case OPT_NUM_OPTS:
	case OPT_TEST:
	case OPT_DEPTH:
	case OPT_BRIGHTNESS:
	case OPT_CONTRAST:
	case OPT_BLACK_LEVEL:
	case OPT_WHITE_LEVEL:
	case OPT_HUE:
	case OPT_SATURATION:
	  s->val[option].w = *(SANE_Word *) val;
	  return SANE_STATUS_GOOD;

	  /* options with side-effects: */
	case OPT_RESOLUTION:
	  if (info)
	    *info |= SANE_INFO_RELOAD_OPTIONS | SANE_INFO_RELOAD_PARAMS;

	  /* low-resolution mode: */
	  s->resolution = QC_RES_LOW;
	  if (strcmp (val, resolution_list[QC_RES_HIGH]) == 0)
	    /* high-resolution mode: */
	    s->resolution = QC_RES_HIGH;
	  s->opt[OPT_TL_X].constraint.range = &x_range[s->resolution];
	  s->opt[OPT_BR_X].constraint.range = &odd_x_range[s->resolution];
	  s->opt[OPT_TL_Y].constraint.range = &y_range[s->resolution];
	  s->opt[OPT_BR_Y].constraint.range = &odd_y_range[s->resolution];

	  if (!(s->user_corner & 0x1))
	      if (s->resolution == QC_RES_LOW)
		s->val[OPT_TL_X].w = 10;
	      else
		s->val[OPT_TL_X].w = 20;
	  if (!(s->user_corner & 0x4))
	      s->val[OPT_BR_X].w = odd_x_range[s->resolution].max;
	  if (!(s->user_corner & 0x8))
	      s->val[OPT_BR_Y].w = odd_y_range[s->resolution].max - 4;
	  return SANE_STATUS_GOOD;
	}
    }
  return SANE_STATUS_INVAL;
}

SANE_Status
sane_get_parameters (SANE_Handle handle, SANE_Parameters *params)
{
  QC_Scanner *s = handle;
  QC_Device *q = s->hw;
  int xfer_scale = s->val[OPT_XFER_SCALE].w;
  size_t Bpp = 3;	/* # of bytes per pixel */

  s->params.format = SANE_FRAME_RGB;
  if (q->version == QC_MONO)
    {
      s->params.format = SANE_FRAME_GRAY;
      Bpp = 1;
    }
  s->params.first_frame = SANE_TRUE;

  s->params.pixels_per_line = s->val[OPT_BR_X].w - s->val[OPT_TL_X].w + 1;
  s->params.lines = s->val[OPT_BR_Y].w - s->val[OPT_TL_Y].w + 1;

  s->params.pixels_per_line /= xfer_scale;
  s->params.lines /= xfer_scale;
  s->params.bytes_per_line = Bpp * s->params.pixels_per_line;
  s->params.depth = 8;

  if (params)
    *params = s->params;

  return SANE_STATUS_GOOD;
}

SANE_Status
sane_start (SANE_Handle handle)
{
  int top, left, width, height, undecimated_width, undecimated_height;
  QC_Scanner *s = handle;
  QC_Device *q = s->hw;

  sane_get_parameters (s, 0);		/* ensure uptodate parameters */

  qc_lock (q);

  if (q->version == QC_COLOR)
    {
      int black_level;

      qc_send (q, QC_SET_SPEED);
      qc_send (q, 1);	/* 2 is official, but 1 works fine */

      /* wait for camera to become ready: */
      while (qc_getstatus (q) & CameraNotReady)
	usleep (10000);

      /* Only send black_level if necessary; this optimization may
	 fail if two applications access the camera in an interleaved
	 fashion; but the black-level command is slow enough that it
	 cannot be issued for every image acquisition.  */
      black_level = s->val[OPT_BLACK_LEVEL].w;
      if (black_level != s->previous_black_level)
	{
	  DBG(QCAM, 3, "sane_start: black_level=%d\n", black_level);

	  qc_send (q, QC_SET_BLACK);
	  qc_send (q, black_level);

	  /* wait for set black level command to finish: */
	  while (qc_getstatus (q) & CameraNotReady)
	    usleep (10000);

	  s->previous_black_level = black_level;
	}

      qc_send (q, QC_COL_SET_HUE);
      qc_send (q, s->val[OPT_HUE].w);

      qc_send (q, QC_SET_SATURATION);
      qc_send (q, s->val[OPT_SATURATION].w);
  }

  qc_send (q, QC_SET_CONTRAST);
  qc_send (q, s->val[OPT_CONTRAST].w);

  qc_send (q, QC_SET_BRIGHTNESS);
  qc_send (q, s->val[OPT_BRIGHTNESS].w);

  width  = s->params.pixels_per_line;
  height = s->params.lines;
  if (s->resolution == QC_RES_HIGH)
    {
      width  /= 2;	/* the expansion occurs through oversampling */
      height /= 2;	/* we acquire only half the lines that we generate */
    }
  undecimated_width  = width *  s->val[OPT_XFER_SCALE].w;
  undecimated_height = height * s->val[OPT_XFER_SCALE].w;

  s->num_bytes = s->x = s->y = 0;
  s->bytes_per_frame = width * height;
  if (q->version == QC_COLOR)
    {
      if (s->resolution == QC_RES_LOW)
	s->bytes_per_frame *= 3;
      else
	s->bytes_per_frame *= 4;
    }

  qc_send (q, QC_SET_NUM_V);
  qc_send (q, undecimated_height);

  if (q->version == QC_COLOR)
    {
      qc_send (q, QC_SET_NUM_H);
      qc_send (q, undecimated_width / 2);
    }
  else
    {
      int val;

      if (q->port_mode == QC_UNIDIR && s->val[OPT_DEPTH].w == 6)
	val = undecimated_width;
      else
	val = undecimated_width * 4;
      qc_send (q, QC_SET_NUM_H);
      qc_send (q, val);
  }

  left = s->val[OPT_TL_X].w / 2;
  top  = s->val[OPT_TL_Y].w;
  if (s->resolution == QC_RES_HIGH)
    {
      left /= 2;
      top  /= 2;
    }

  DBG(QCAM, 3, "sane_start: top=%d, left=%d, white=%d, bright=%d, contr=%d\n",
      top, left, s->val[OPT_WHITE_LEVEL].w, s->val[OPT_BRIGHTNESS].w,
      s->val[OPT_CONTRAST].w);

  qc_send (q, QC_SET_LEFT);
  qc_send (q, left);

  qc_send (q, QC_SET_TOP);
  qc_send (q, top + 1);		/* not sure why this is so... ;-( */

  qc_send (q, QC_SET_WHITE);
  qc_send (q, s->val[OPT_WHITE_LEVEL].w);

  qc_setscanmode (s);

  DBG(QCAM, 2,
      "qcam.start: %s %d lines of %d pixels each (%d bytes) => %dx%d\n",
      (q->port_mode == QC_BIDIR) ? "bidir" : "unidir",
      height, width, s->bytes_per_frame, s->params.pixels_per_line,
      s->params.lines);

  s->scanning = SANE_TRUE;
  s->buffer_valid = SANE_FALSE;
  return SANE_STATUS_GOOD;
}

SANE_Status
sane_read (SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len,
	   SANE_Int *lenp)
{
  QC_Scanner *s = handle;
  QC_Device *q = s->hw;
  size_t n, bytes, len;
  SANE_Byte *buf_end;

  if (!s->buffer_valid)
    {
      if (s->buffer)
	s->buffer = realloc (s->buffer, s->bytes_per_frame);
      else
	s->buffer = malloc (s->bytes_per_frame);

      /* Don't do this in sane_start() since there may be a long
	 timespan between it and the first sane_read(), which would
	 result in poor images.  */
      qc_send (q, QC_SEND_VIDEO_FRAME);
      qc_send (q, s->mode);

      if (q->port_mode == QC_BIDIR)
	{
	  /* turn port into input port */
	  write_lpcontrol (q, Autofeed | Reset_N | PCAck | BiDir);
	  usleep (3);
	  write_lpcontrol (q, Autofeed | Reset_N |	   BiDir);
	  qc_waithand (q, CamRdy1);
	  write_lpcontrol (q, Autofeed | Reset_N | PCAck | BiDir);
	  qc_waithand (q, 0);
	}

      if (q->version == QC_COLOR)
	{
	  for (len = 0; len < s->bytes_per_frame; len += bytes)
	    {
	      bytes = qc_readbytes (s, s->buffer + len);
	      assert (bytes == 3);
	      if (!s->scanning)
		goto unlock_and_cancel;
	    }
	}
      else
	{
#ifdef XXX
	  /* strange -- should be 15:63 below, but 4bpp is odd */
	  invert = (s->val[OPT_DEPTH] == 4) ? 16 : 63;

	  for (i = 0; i < linestotrans; i++)
	    {
	      for (pixels_read = j = 0; j < transperline; j++)
		{
		  bytes = qc_readbytes (q, buffer);
		  assert (bytes > 0);
		  for (k = 0; k < bytes && (pixels_read + k) < pixels_per_line; k++)
		    {
		      assert (buffer[k] <= invert);
		      if (buffer[k] == 0 && invert == 16)
			{
			  /* 4bpp is odd (again) -- inverter is 16, not
			     15, but output must be 0-15 */
			  buffer[k] = 16;
			}
		      ret[i*pixels_per_line + pixels_read + k] = invert - buffer[k];
		    }
		  pixels_read += bytes;
		}
	      qc_readbytes (q, 0); /* reset state machine */
	    }
#endif
	}
      /* we're done with this frame: */
      DBG(QCAM, 2, "qcam.read: frame complete\n");

      if (q->port_mode == QC_BIDIR)
	{
	  /* return port to output mode */
	  write_lpcontrol (q, Autofeed);
	  usleep (3);
	  write_lpcontrol (q, Autofeed | Reset_N);
	  usleep (3);
	  write_lpcontrol (q, Autofeed | Reset_N | PCAck);
	}

      qc_unlock (q);

      s->buffer_valid = SANE_TRUE;
    }
  else if (s->num_bytes >= s->bytes_per_frame)
    {
      s->buffer_valid = SANE_FALSE;
      s->scanning = SANE_FALSE;
      return SANE_STATUS_EOF;
    }

  if (s->resolution == QC_RES_HIGH)
    {
      SANE_Byte *src, *buf_start = buf, *buf_end = buf + max_len - 5;
      int x, y;

      /* in billions mode, we need to oversample the data: */
      x = s->x;
      y = s->y;
      src = s->buffer + s->num_bytes;
      while (buf < buf_end)
	{
	  int red1, green1, blue1, red2, green2, blue2, green3, blue3, blue4;

	  blue1 = src[0]; green1 = src[1]; red1 = src[3];

	  if (y & 1)
	    {
	      /* odd line */
	      green3 = src[2 * s->params.pixels_per_line + 1];
	      blue3  = src[2 * s->params.pixels_per_line];
	      blue4  = src[2 * s->params.pixels_per_line + 4];

	      buf[0] = red1;
	      buf[1] = green3;
	      buf[2] = blue3;
	      buf[3] = red1;
	      buf[4] = green3;
	      buf[5] = blue4;
	    }
	  else
	    {
	      /* even line */
	      blue2 = src[4]; green2 = src[5]; red2 = src[7];
	      buf[0] = red1;
	      buf[1] = green1;
	      buf[2] = blue1;
	      buf[3] = red1;
	      buf[4] = green1;
	      buf[5] = blue2;
	    }
	  src += 4;
	  buf += 6;
	  x += 2;
	  if (x >= s->params.pixels_per_line)
	    {
	      if (++y & 1)
		src -= 2*s->params.pixels_per_line;
	      x = 0;
	      if (y >= s->params.lines)
		break;
	    }
	}
      s->num_bytes = src - s->buffer;;
      s->x = x;
      s->y = y;
      len = buf - buf_start;
    }
  else
    {
      len = s->bytes_per_frame - s->num_bytes;
      if (len > max_len)
	len = max_len;

      memcpy (buf, s->buffer + s->num_bytes, len);
      s->num_bytes += len;
    }
  if (lenp)
    *lenp = len;

  return SANE_STATUS_GOOD;

unlock_and_cancel:
  qc_unlock (q);
  return SANE_STATUS_CANCELLED;
}

void
sane_cancel (SANE_Handle handle)
{
  QC_Scanner *s = handle;

  s->scanning = SANE_FALSE;
}

SANE_Status
sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
{
  QC_Scanner *s = handle;

#ifdef XXX
  if (!s->scanning)
    return SANE_STATUS_INVAL;

  if (fcntl (s->pipe, F_SETFL, non_blocking ? O_NONBLOCK : 0) < 0)
    return SANE_STATUS_IO_ERROR;
  return SANE_STATUS_GOOD;
#endif
  return SANE_STATUS_UNSUPPORTED;
}

SANE_Status
sane_get_select_fd (SANE_Handle handle, SANE_Int *fd)
{
  QC_Scanner *s = handle;

#ifdef XXX
  if (!s->scanning)
    return SANE_STATUS_INVAL;

  *fd = s->pipe;
  return SANE_STATUS_GOOD;
#endif
  return SANE_STATUS_UNSUPPORTED;
}
